# Getting Started

## Structure

The SDK contains headers that you need for building a plugin that can interact and modify the UI of CapturyLive or CapturyStudio. It also contains some sample plugins.

## Prepare

install `cmake`, `gcc/clang` (incl. C++ support) and optionally `qt5base-dev`

## Build

```
tar -xf CapturyLiveSDK.tar.gz
cd sdk
mkdir build && cd build
cmake ../plugins
make
```
Then copy the lib*Plugin.so into the folder where CapturyLive is (the bin folder). Next time CapturyLive starts it should start these plugins as well.

## Hack

To build your own plugin you just need to build a library that provides a function `initPlugin`

```
// provided by the plugin
// fill info structure and return 1 if successful
CAPTURY_DLL_EXPORT unsigned char initPlugin(pluginInfo* info);
```

`info` is passed to the plugin. The host application expects that the plugin fills in the details that it is interested in. In particular, some callback functions can be registered. The declaration and all functions that are provided by the host application are declared in `plugin.h`.

Note that there are two ways to access the host functionality. The function `liveGetHostSuite` allows the plugin to retrieve sets of related function pointers for example for logging, which can then be used to push a message into the logging system of the host application. Typically this would be used like so:

```
static TCLogHostSuite* logging;
unsigned char initPlugin(pluginInfo* info)
{
	logging = (TCLogHostSuite*)info->getHostSuite("Log");

	logging->info("hello world!");
}
```

This is the preferred method of accessing host functionality. Unfortunately, not all functions have been ported yet. So the old API is still available and required. There are a lot of functions at the bottom of the header. Please let us know if any functionality you need is missing.
