#pragma once

#if defined WIN32 || defined _WIN32 && !defined __CYGWIN__
  #define NOMINMAX
#else
  #define sprintf_s	snprintf
  #define vsprintf_s	vsnprintf
  #ifndef __has_extension
    #define __has_extension	__has_feature
  #endif
  #ifndef __has_feature
    #define __has_feature(x)	0
  #endif
  #if (defined __clang__ && !__has_feature(cxx_nullptr) && !__has_extension(cxx_nullptr))
    #define nullptr __null
  #endif
  #if (!defined __clang__ && !defined nullptr && (((__GNUC__ * 100) + __GNUC_MINOR__) < 407)) || defined __CUDA_ARCH__
    #define nullptr	__null
  #endif
#endif

#if defined WIN32 || defined _WIN32 && !defined __CYGWIN__
  #include <Windows.h>
  #include <inttypes.h>

  static inline int64_t  atomicDecrement(int64_t* ptr)  { return InterlockedDecrement64(ptr); }
  static inline LONG     atomicDecrement(LONG* ptr)     { return InterlockedDecrement(ptr); }
  static inline int32_t  atomicDecrement(int32_t* ptr)  { return InterlockedDecrement((volatile LONG*)ptr); }
  static inline int64_t  atomicIncrement(int64_t* ptr)  { return InterlockedIncrement64(ptr); }
  static inline LONG     atomicIncrement(LONG* ptr)     { return InterlockedIncrement(ptr); }
  static inline int32_t  atomicIncrement(int32_t* ptr)  { return InterlockedIncrement((volatile LONG*)ptr); }
  static inline uint32_t atomicIncrement(uint32_t* ptr) { return InterlockedIncrement((volatile LONG*)ptr); }
  static inline void atomicAddInt(int64_t* ptr, int64_t value)  { InterlockedAdd64(ptr, value); }
  static inline void atomicAddInt(LONG* ptr, LONG value)  { InterlockedAdd(ptr, value); }
  static inline void atomicStore(int64_t* ptr, int64_t value) { InterlockedExchange64(ptr, value); }
  static inline void atomicStore(void** ptr, void* value) { InterlockedExchange64((intptr_t*)ptr, (intptr_t)value); }
  static inline void atomicStore(LONG* ptr, LONG value)  { InterlockedExchange(ptr, value); }
  static inline void atomicLoad(int64_t* srcPtr, int64_t* dstPtr) { *dstPtr = InterlockedOr64(srcPtr, 0); }
  static inline void atomicLoad(void** srcPtr, void** dstPtr) { *dstPtr = (void*)InterlockedOr64((intptr_t*)srcPtr, 0); }
  static inline void atomicLoad(LONG* srcPtr, LONG* dstPtr)  { *dstPtr = InterlockedOr(srcPtr, 0); }
  static inline void atomicExchange(int64_t* dstPtr, int64_t* srcPtr)  { InterlockedExchange64(srcPtr, *srcPtr); }
  static inline void atomicExchange(LONG* dstPtr, LONG* srcPtr)  { InterlockedExchange(srcPtr, *srcPtr); }
  static inline void atomicCompareExchange(int64_t* dstPtr, int64_t* expectPtr, int64_t* srcPtr)  { *expectPtr = InterlockedCompareExchange64(dstPtr, *srcPtr, *expectPtr); }
  static inline void atomicCompareExchange(LONG* dstPtr, LONG* expectPtr, LONG* srcPtr)	 { *expectPtr = InterlockedCompareExchange(dstPtr, *srcPtr, *expectPtr); }

  static inline bool mul_overflow(int64_t a, int64_t b, int64_t* out)
  {
    int64_t high;
    *out = _mul128(a, b, &high);
    return (high != 0);
  }
  static inline bool add_overflow(int64_t a, int64_t b, int64_t* out)
  {
    *out = a + b;
    return false;
  }
  static inline bool sub_overflow(int64_t a, int64_t b, int64_t* out)
  {
    *out = a - b;
    return false;
  }
#else
  #define atomicDecrement(ptr)		__sync_sub_and_fetch(ptr, 1)
  #define atomicIncrement(ptr)		__sync_add_and_fetch(ptr, 1)
  #define atomicAddInt(ptr, value)		__sync_add_and_fetch(ptr, value)
  #if ((__GNUC__ * 100 + __GNUC_MINOR__ * 10) > 464)
    #define atomicStore(dstPtr, value)	__atomic_store(dstPtr, &(value), __ATOMIC_RELAXED)
    #define atomicLoad(srcPtr, dstPtr)		__atomic_load(srcPtr, dstPtr, __ATOMIC_RELAXED)
    #define atomicExchange(dstPtr, srcPtr, origValPtr)		__atomic_exchange(dstPtr, srcPtr, origValPtr, __ATOMIC_RELAXED)
    #define atomicCompareExchange(dstPtr, expectPtr, srcPtr)		__atomic_compare_exchange(dstPtr, expectPtr, srcPtr, true, __ATOMIC_RELAXED, __ATOMIC_RELAXED)
  #else
    #define atomicStore(where, what)	*(where) = (what);
    #define atomicLoad(srcPtr, dstPtr)		*(dstPtr) = *(srcPtr);
    #define atomicExchange(dstPtr, srcPtr, origValPtr)		*(origValPtr) = *(dstPtr); *(dstPtr) = *(srcPtr);
    #define atomicCompareExchange(dstPtr, expectPtr, srcPtr)		{ if (*(dstPtr) == *(expectPtr)) *(dstPtr) = *(srcPtr); else *(expectPtr) = *(srcPtr); }
  #endif
  #if __LONG_MAX__ > 0xFFFFFFFF
    #define mul_overflow  __builtin_smull_overflow
    #define add_overflow  __builtin_saddl_overflow
    #define sub_overflow  __builtin_ssubl_overflow
  #else
    #define mul_overflow  __builtin_smulll_overflow
    #define add_overflow  __builtin_saddll_overflow
    #define sub_overflow  __builtin_ssubll_overflow
  #endif
#endif
