#pragma once

#include "captury/types.h"

#include "captury/PublicStructs.h"

#ifdef WIN32
#define CAPTURY_DLL_EXPORT __declspec(dllexport)
#else
#define CAPTURY_DLL_EXPORT
#endif

#define DOCK_LEFT	1
#define DOCK_RIGHT	2
#define DOCK_TOP	4
#define DOCK_BOTTOM	8

enum LiveActorMode { liveSkeletonPreloaded, liveSkeletonInactive, liveSkeletonSearching, liveSkeletonSearchingForRecolor, liveSkeletonScaling, liveSkeletonTracking, liveSkeletonPlayback };

enum LiveShot { liveActiveShot = 0, liveLiveShot = 1 };

enum DeviceClass { liveDeviceCamera, liveDeviceIMU, liveDeviceADBoard };
enum DeviceStatus { liveDeviceDisconnected = 0, liveDeviceConnecting, liveDeviceConnected, liveDeviceStarting, liveDeviceRunning, liveDeviceLost };

enum LivePhysiologicalAngleType {
	liveLeftKneeFlexionExtension,
	liveLeftKneeFlexion,
	liveLeftKneeExtension,
	liveLeftKneeVarusValgus,
	liveLeftKneeVarus,
	liveLeftKneeValgus,
	liveLeftKneeRotation, // both internal and external
	liveLeftKneeInternalRotation,
	liveLeftKneeExternalRotation,
	liveLeftHipFlexionExtension,
	liveLeftHipFlexion,
	liveLeftHipExtension,
	liveLeftHipAbAdduction, // both ab- and adduction
	liveLeftHipAbduction,
	liveLeftHipAdduction,
	liveLeftHipRotation, // both internal and external
	liveLeftHipInternalRotation,
	liveLeftHipExternalRotation,
	liveLeftAnkleFlexionExtension,
	liveLeftAnkleFlexion,
	liveLeftAnkleExtension,
	liveLeftAnklePronationSupination,
	liveLeftAnklePronation,
	liveLeftAnkleSupination,
	liveLeftAnkleRotation,
	liveLeftAnkleInternalRotation,
	liveLeftAnkleExternalRotation,
	liveLeftShoulderFlexionExtension,
	liveLeftShoulderFlexion,
	liveLeftShoulderExtension,
	liveLeftShoulderTotalFlexion,
	liveLeftShoulderAbAdduction, // both ab- and adduction
	liveLeftShoulderAbduction,
	liveLeftShoulderAdduction,
	liveLeftShoulderRotation,
	liveLeftShoulderInternalRotation,
	liveLeftShoulderExternalRotation,
	liveLeftElbowFlexionExtension,
	liveLeftElbowFlexion,
	liveLeftElbowExtension,
	liveLeftForearmPronationSupination,
	liveLeftForearmPronation,
	liveLeftForearmSupination,
	liveLeftWristFlexionExtension,
	liveLeftWristFlexion,
	liveLeftWristExtension,
	liveLeftWristRadialUlnarDeviation,
	liveLeftWristRadialDeviation,
	liveLeftWristUlnarDeviation,
	liveRightKneeFlexionExtension,
	liveRightKneeFlexion,
	liveRightKneeExtension,
	liveRightKneeVarusValgus,
	liveRightKneeVarus,
	liveRightKneeValgus,
	liveRightKneeRotation, // both internal and external
	liveRightKneeInternalRotation,
	liveRightKneeExternalRotation,
	liveRightHipFlexionExtension,
	liveRightHipFlexion,
	liveRightHipExtension,
	liveRightHipAbAdduction, // both ab- and adduction
	liveRightHipAbduction,
	liveRightHipAdduction,
	liveRightHipRotation, // both internal and external
	liveRightHipInternalRotation,
	liveRightHipExternalRotation,
	liveRightAnkleFlexionExtension,
	liveRightAnkleFlexion,
	liveRightAnkleExtension,
	liveRightAnklePronationSupination,
	liveRightAnklePronation,
	liveRightAnkleSupination,
	liveRightAnkleRotation,
	liveRightAnkleInternalRotation,
	liveRightAnkleExternalRotation,
	liveRightShoulderFlexionExtension,
	liveRightShoulderFlexion,
	liveRightShoulderExtension,
	liveRightShoulderTotalFlexion,
	liveRightShoulderAbAdduction, // both ab- and adduction
	liveRightShoulderAbduction,
	liveRightShoulderAdduction,
	liveRightShoulderRotation,
	liveRightShoulderInternalRotation,
	liveRightShoulderExternalRotation,
	liveRightElbowFlexionExtension,
	liveRightElbowFlexion,
	liveRightElbowExtension,
	liveRightForearmPronationSupination,
	liveRightForearmPronation,
	liveRightForearmSupination,
	liveRightWristFlexionExtension,
	liveRightWristFlexion,
	liveRightWristExtension,
	liveRightWristRadialUlnarDeviation,
	liveRightWristRadialDeviation,
	liveRightWristUlnarDeviation,
	liveNeckFlexionExtension,
	liveNeckFlexion,
	liveNeckExtension,
	liveNeckRotation,
	liveNeckLateralBending,
	liveCenterOfGravityX,
	liveCenterOfGravityY,
	liveCenterOfGravityZ,
	liveFrontalPlaneKneeAngle,
	liveFrontalPlaneProjectionAngle,
	liveToeIn,
	liveFunctionalReach, // distance from center of mass to wrist
	livePelvisDrop,
	livePelvisTilt,
	liveHeadRotation,
	liveHeadRotationLeft,
	liveHeadRotationRight,
	liveTorsoRotation,
	liveTorsoInclination,
	liveHeadInclination,
	liveTorsoFlexion,
	livePhysiologicalAngleTypeCount
};

struct Fraction {
	int64_t		numerator;
	uint64_t	denominator;
};

#ifdef _MSC_VER
  #define __attribute__(a)
#endif

#pragma pack(push,1)

struct TCHostSuite {
	void* host;
	void* (*getHostSuite)(const char* name); // "Memory", "Log", "Camera", "Settings", "Stream"
	int (*registerPluginSuite)(const char* name, void* suite); // "Camera", "Stream", "UI"
};

struct TCMemoryHostSuite {
	void* (*malloc)(size_t numBytes);
	void (*free)(void* ptr);
};

struct TCLogHostSuite {
	void (*info)(const char* format, ...) __attribute__((format(printf, 1, 2)));
	void (*warning)(const char* format, ...) __attribute__((format(printf, 1, 2)));
	void (*error)(const char* format, ...) __attribute__((format(printf, 1, 2)));
};

struct TCSettingsHostSuite {
	int (*storeString)(void* host, const char* name, const char* value);
	int (*storeInt)(void* host, const char* name, int value);
	int (*storeDouble)(void* host, const char* name, double value);
	char* (*retrieveString)(void* host, const char* name, char* str, int size, const char* defaultValue); // returns str on success
	int (*retrieveInt)(void* host, const char* name, int defaultValue);
	double (*retrieveDouble)(void* host, const char* name, double defaultValue);
};

struct TCStreamHostSuite {
	int (*getNumActors)(int liveShot);
	int (*getActorIds)(int numActorIds, int* ids, int liveShot);
	CapturyActor* (*getActor)(int actorId);	// MemoryHostSuite->free() after use. don't worry about joints. they will be freed automatically
	LiveActorMode (*getActorMode)(int actorId);
	CapturyPose* (*getCurrentPose)(int actorId); // MemoryHostSuite->free() after use.

	int (*getNumCameras)(int liveShot);
	int (*getCameraIds)(int numCamIds, uint* ids, int liveShot);
	int (*getCurrentCameraId)();
	CapturyCamera* (*getCamera)(uint id);

	// pass in buffer large enough to hold the pose, *numDofs is updated to the actually used space
	float* (*getCurrentRawDofsPose)(int actorId, int liveShot, float* dofs, int* numDofs);
};

struct TCConfigHostSuite {
	void (*addInt)      (void* host, const char* name, const char* label, int min, int max, int step);
	void (*addDouble)   (void* host, const char* name, const char* label, double min, double max, double step);
	void (*addFlag)     (void* host, const char* name, const char* label);
	void (*addButton)   (void* host, const char* name, const char* label);
	void (*addText)     (void* host, const char* name, const char* label);
	void (*addLine)     (void* host, const char* name, const char* label);
	void (*addIPAddress)(void* host, const char* name, const char* label);

	int (*getInt)(void* plugin, const char* name);
	double (*getDouble)(void* plugin, const char* name);
	int (*isFlagSet)(void* plugin, const char* name);
	void (*getTextEditText)(void* plugin, const char* name, char* buffer, int size);
	void (*getLineText)(void* plugin, const char* name, char* buffer, int size);
	void (*getIPAddress)(void* plugin, const char* name, char* buffer, int size);

	void (*setInt)(void* plugin, const char* name, int v);
	void (*setDouble)(void* plugin, const char* name, double v);
	void (*setFlag)(void* plugin, const char* name, int yes);
	void (*setButtonText)(void* plugin, const char* name, const char* text);
	void (*setLine)(void* plugin, const char* name, const char* text);
};

struct TCChartHostSuite {
	int (*selectChart)(const char* chartName);
	int (*selectChartSkeleton)(int skelId);
	float (*getCurrentChartValue)(int skelId, LivePhysiologicalAngleType type);
};

// for plugins that implement a device interface
struct TCDeviceHostSuite {
	void (*foundDevice)(DeviceClass type, uint id, const char* name);

	void (*pushImage)(CapturyImage* img);
	void (*pushPose)(CapturyIMUPose* pose);

	uint64_t (*getTime)(); // in microseconds
};

struct TCDevicePluginSuite {
	int (*discover)();
	int (*watch)();

	const char* (*getHostname)(uint id);
	const char* (*getMake)(uint id);
	const char* (*getModel)(uint id);
	const char* (*getSerialNumber)(uint id);

	int (*reset)(uint id);

	int (*connect)(uint id);
	int (*disconnect)(uint id);
	int (*isConnected)(uint id);

	int (*start)(uint id);
	int (*stop)(uint id);
	int (*isRunning)(uint id);

	int (*getTemperature)(float* t);

	DeviceStatus (*getStatus)(uint id);

	Fraction (*getFramerate)(uint id);
	Fraction (*getMaximumFramerate)(uint id);
};

struct TCCameraPluginSuite {

};

struct TCStreamPluginSuite {
	void (*actorModeChanged)(int actorId, enum LiveActorMode);
	void (*newPose)(CapturyPose* pose, int trackingQuality);

	void (*newVideoFrame)(const CapturyImage* img);
	void (*newARTag)(int numTags, CapturyARTag* tags);
	void (*newCalibrationBoard)(int frame, int num, const CapturyCornerDetection* corners);
	void (*editorCameraChanged)(const CapturyCamera* camera);
};

#pragma pack(pop)

// provided by the plugin
// return name of plugin or NULL if failed
CAPTURY_DLL_EXPORT const char* tcInitPlugin(TCHostSuite* plugin);

// LEGACY INTERFACE
// DO NOT USE

struct pluginInfo {
	int		size;			// size of this structure

	const char*	name;

	int		activeInLiveMode;
	int		activeInPlaybackMode;

	int		defaultDockArea;

	// final cleanup. free memory, etc.
	void (*deinit)(pluginInfo*);

	// start&stop processing thread if needed
	void (*activate)(pluginInfo* info);
	void (*deactivate)();

	// use OpenGL to paint into the main window
	void (*paint)();

	// size is the size of the buffer (number of elements) that can be filled by the plugin
	// returns the number of constraints it set
	// this callback is optional. set to NULL if you don't want to provide constraints to CapturyLive.
	uint (*getConstraints)(uint size, CapturyConstraint* buffer);

	// called when one of the knobs changes or is pressed
	void (*knobChanged)(const char* name);

	// called when the selected shot changes. name is empty when the current shot is the live shot
	void (*shotChanged)(const char* oldPath, const char* newPath);

	// called when the selected frame changes. will be called all the time if the shot is a live shot.
	void (*frameChanged)(int frame, int isLiveShot);

	// called when a new actor is tracked/scaled/stopped
	void (*actorModeChanged)(uint64_t actorId, LiveActorMode mode);

	// get host suite
	void* (*getHostSuite)(const char* name);
};

#ifdef __cplusplus
extern "C" {
#endif
	// provided by the plugin
	// fill info structure and return 1 if successful
	CAPTURY_DLL_EXPORT unsigned char initPlugin(pluginInfo* info);

	//
	// provided by the host application
	//

	CAPTURY_DLL_EXPORT void* liveGetHostSuite(const char* name);

	// use this function to free memory allocated by the host application
	CAPTURY_DLL_EXPORT void liveFree(void* ptr);

	CAPTURY_DLL_EXPORT const char* liveGetVersion(); // e.g. "CapturyLive 1.0.118 Crazy Bat"

	// cameras
	CAPTURY_DLL_EXPORT uint liveGetNumCameras(int liveShot);
	CAPTURY_DLL_EXPORT uint liveGetCameraIds(uint numCamIds, uint64_t* camIds, int liveShot);
	CAPTURY_DLL_EXPORT uint64_t liveGetCurrentCameraId();
	CAPTURY_DLL_EXPORT void liveSwitchToCamera(uint64_t id);
	CAPTURY_DLL_EXPORT CapturyCamera* liveGetCamera(uint64_t id, int liveShot);

	// returns the extrinsic calibration error and intrinsic reconstruction errors in intrinsicError[]
	// the caller needs to preallocate intrinsicErrors
	CAPTURY_DLL_EXPORT float liveGetCalibrationAccuracy(float* intrinsicError, int liveShot);

	CAPTURY_DLL_EXPORT int liveGetWhiteBalance(uint64_t id, float* red, float* blue);
	CAPTURY_DLL_EXPORT int liveSetWhiteBalance(uint64_t id, float red, float blue);
	CAPTURY_DLL_EXPORT float liveGetBrightness(uint64_t id);
	CAPTURY_DLL_EXPORT int liveSetBrightness(uint64_t id, float brightness);

	// min and max are arrays of 3 floats each for [x,y,z]
	CAPTURY_DLL_EXPORT void liveGetTrackingVolume(float* min, float* max, int liveShot);

	// the plugin is responsible for freeing the buffer - use liveFree() to do so
	CAPTURY_DLL_EXPORT void liveGetCurrentImage(uint64_t camId, uint* size, unsigned char** buffer);

	// returns the frame number or -1 if the frame cannot be determined (e.g. in live mode)
	CAPTURY_DLL_EXPORT void liveGetCurrentScreen(uint width, uint height, uint* size, unsigned char** buffer);

	// returns the frame number or -1 if the frame cannot be determined (e.g. in live mode)
	CAPTURY_DLL_EXPORT int liveGetCurrentFrame();

	// returns -1 if the frame cannot be set (e.g. in live mode)
	CAPTURY_DLL_EXPORT int liveSetCurrentFrame(int frame);

	// actors
	CAPTURY_DLL_EXPORT uint liveGetNumActors(int liveShot);
	CAPTURY_DLL_EXPORT int liveGetActorIds(uint numActorIds, uint64_t* actorIds, int liveShot);
	CAPTURY_DLL_EXPORT int liveGetActorNames(uint numActorIds, char** actorNames, int liveShot); // pass array of pointers to preallocated buffers [32] bytes each
	CAPTURY_DLL_EXPORT CapturyActor* liveGetActor(uint64_t actorId, int liveShot);		// do not free!
	CAPTURY_DLL_EXPORT CapturyPose* liveGetCurrentPose(uint64_t actorId, int liveShot);	// liveFree() after use.
	CAPTURY_DLL_EXPORT CapturyImage* liveGetTexture(uint64_t actorId, int liveShot);	// liveFree() after use.
	CAPTURY_DLL_EXPORT void liveSetActorHeightRange(float minHeightInMM, float maxHeightInMM);
	// snapMode must be BackgroundSubtractionLocal : 0, BackgroundSubtractionGlobal : 1, BodyPartDetectorLocal : 2, BodyPartDetectorGlobal : 3, BodyPartDetectorJoints : 4
	CAPTURY_DLL_EXPORT void liveSnapActor(float x, float z, float radius, float heading, const char* skeletonName, int snapMode, int quickScaling);
	CAPTURY_DLL_EXPORT void liveStartSearching();
	CAPTURY_DLL_EXPORT void liveStartTracking(uint64_t actorId, float x, float z, float heading);
	CAPTURY_DLL_EXPORT void liveRescaleActor(uint64_t actorId);
	CAPTURY_DLL_EXPORT void liveRecolorActor(uint64_t actorId, bool updateColors);
	CAPTURY_DLL_EXPORT void liveStopTracking(uint64_t actorId);
	CAPTURY_DLL_EXPORT void liveDeleteActor(uint64_t actorId);
	CAPTURY_DLL_EXPORT void liveSetActorName(uint64_t actorId, const char* name); // always works on the live shot - actors in playback mode cannot be renamed
	CAPTURY_DLL_EXPORT void liveSetActorMetaData(uint64_t actorId, const char* name, const char* value); // always works on the live shot - actors in playback mode cannot be modified
	CAPTURY_DLL_EXPORT const char* liveGetActorMetaData(uint64_t actorId, const char* name); // always works on the live shot - actors in playback mode cannot be modified
	CAPTURY_DLL_EXPORT void liveGetLatency(uint64_t* firstImagePacket, uint64_t* optimizationStart, uint64_t* optimizationEnd);

	// actorJoint indexes into CapturyActor joints. The return value is an index to the CapturyLive internal joint index or -1 if actorId or actorJoint are invalid.
	CAPTURY_DLL_EXPORT int liveMapJoint(uint64_t actorId, int actorJoint);

	// actorJoint indexes into CapturyActor joints. The function returns the transform of the specific joint computed by tracked and 1 if everything goes according to plan.
	// optionally also outputs the timestamp the transform was estimated at.
	CAPTURY_DLL_EXPORT int liveGetMarkerTransform(uint64_t actorId, int actorJoint, CapturyTransform* transform, uint64_t* timestamp);
	CAPTURY_DLL_EXPORT int liveSetMarkerTransform(uint64_t actorId, int actorJoint, const CapturyTransform* transform, uint64_t timestamp);

	CAPTURY_DLL_EXPORT LiveActorMode liveGetActorMode(uint64_t actorId, int liveShot);
	CAPTURY_DLL_EXPORT int liveGetScalingProgress(uint64_t actorId, int liveShot);
	CAPTURY_DLL_EXPORT int liveGetTrackingQuality(uint64_t actorId, int liveShot);

	// calibration
	CAPTURY_DLL_EXPORT void liveStartDetectingIntrinsicsBoard();
	CAPTURY_DLL_EXPORT void liveStopDetectingIntrinsicsBoard();
	CAPTURY_DLL_EXPORT void liveCalibrateIntrinsics();
	CAPTURY_DLL_EXPORT void liveCancelIntrinsicsCalibration();

	CAPTURY_DLL_EXPORT void liveStartDetectingExtrinsicsBoard();
	CAPTURY_DLL_EXPORT void liveStopDetectingExtrinsicsBoard();
	CAPTURY_DLL_EXPORT void liveClearExtrinsicsBoards();
	CAPTURY_DLL_EXPORT void liveCalibrateExtrinsics();
	CAPTURY_DLL_EXPORT void liveCancelExtrinsicsCalibration();

	CAPTURY_DLL_EXPORT void liveSetReferenceFrame();
	CAPTURY_DLL_EXPORT void liveCancelSettingReferenceFrame();

	CAPTURY_DLL_EXPORT void liveTransformGlobalCoordinateSystem(const CapturyTransform* trafo);

	CAPTURY_DLL_EXPORT void liveStartCapturingBackground();
	CAPTURY_DLL_EXPORT void liveStopCapturingBackground();
	CAPTURY_DLL_EXPORT void liveStartStopFocusAssist();
	CAPTURY_DLL_EXPORT void liveSetFocusAssist(int on);

	CAPTURY_DLL_EXPORT int liveGetBackgroundQuality();

	// recording
	CAPTURY_DLL_EXPORT void liveGetProjectDirectory(char** dir);
	CAPTURY_DLL_EXPORT void liveSetProjectDirectory(const char* shot);
	CAPTURY_DLL_EXPORT void liveSetShotName(const char* shot);
	CAPTURY_DLL_EXPORT void liveDeleteShot(const char* shot);
	CAPTURY_DLL_EXPORT void liveGetCurrentShotDirectory(char** dir);
	CAPTURY_DLL_EXPORT int64_t liveStartRecording(int recordVideos, int recordMotion);
	CAPTURY_DLL_EXPORT void liveStopRecording();         // returns immediately
	CAPTURY_DLL_EXPORT void liveStopRecordingAndWait();  // waits until writing the shot has finished
	CAPTURY_DLL_EXPORT void liveSwitchToPlayback(int playback); // if 0 switches to live mode

	// callbacks
	// the passed pose is only valid for the duration of the callback. do not free or store pointer.
	typedef void (*livePoseCallback)(const CapturyPose* pose, int trackingQuality);
	CAPTURY_DLL_EXPORT void liveRegisterPoseCallback(livePoseCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterPoseCallback(livePoseCallback callback);

	typedef void (*liveIMUCallback)(const CapturyIMUPose* imuPose, int trackingQuality);
	CAPTURY_DLL_EXPORT void liveRegisterIMUCallback(liveIMUCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterIMUCallback(liveIMUCallback callback);

	// the image that is passed in the callback is only valid for the duration of the callback. do not free or store pointer.
	typedef void (*liveVideoCallback)(const CapturyImage* img);
	CAPTURY_DLL_EXPORT int liveRegisterVideoCallback(uint64_t camId, liveVideoCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterVideoCallback(uint64_t camId);

	// the passed pose is only valid for the duration of the callback. do not free or store pointer.
	typedef void (*liveEditorCameraCallback)(const CapturyCamera* camera);
	CAPTURY_DLL_EXPORT void liveRegisterEditorCameraCallback(liveEditorCameraCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterEditorCameraCallback(liveEditorCameraCallback callback);

	typedef void (*liveARTagCallback)(uint64_t timestamp, int numTags, CapturyARTag* tags);
	CAPTURY_DLL_EXPORT void liveRegisterARTagCallback(liveARTagCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterARTagCallback(liveARTagCallback callback);

	// do not free boards. they are owned by the caller.
	typedef void (*liveBoardDetectionCallback)(int frame, int num, const CapturyCornerDetection* corners);
	CAPTURY_DLL_EXPORT void liveRegisterBoardDetectionCallback(liveBoardDetectionCallback callback);
	CAPTURY_DLL_EXPORT void liveUnregisterBoardDetectionCallback(liveBoardDetectionCallback callback);

	// logging
	CAPTURY_DLL_EXPORT void liveLogInfo(const char* format, ...);
	CAPTURY_DLL_EXPORT void liveLogWarning(const char* format, ...);
	CAPTURY_DLL_EXPORT void liveLogError(const char* format, ...);
	// returns "ok" if everything is cool. otherwise returns one issue per line.
	CAPTURY_DLL_EXPORT char* liveGetStatus(); // liveFree() after use.

	// math conversions
	// quat: w,x,y,z
	// euler: in degrees
	CAPTURY_DLL_EXPORT float* liveQuaternionToMatrix(const float* quat, float* m4x4);
	CAPTURY_DLL_EXPORT float* liveMatrixToQuaternion(const float* m4x4, float* quat);
	CAPTURY_DLL_EXPORT float* liveTransformToMatrix(const CapturyTransform* trafo, float* m4x4);
	CAPTURY_DLL_EXPORT float* liveEulerToMatrix(const float* euler, float* m4x4);
	CAPTURY_DLL_EXPORT float* liveEulerToQuaternion(const float* euler, float* quat);
	CAPTURY_DLL_EXPORT float* liveMatrixToEuler(const float* m4x4, float* euler);
	CAPTURY_DLL_EXPORT float* liveQuaternionToEuler(const float* quat, float* euler);

	CAPTURY_DLL_EXPORT float* liveMatrixVector(const float* m4x4, const float* v3, float* result);
	CAPTURY_DLL_EXPORT float* liveMatrixMatrix(const float* m1, const float* m2, float* result);
	CAPTURY_DLL_EXPORT float* liveInvertMatrix(const float* m4x4, float* inv); // assumes that this is a rigid transform
	CAPTURY_DLL_EXPORT float* liveSwapHandedness(const float* m4x4, float* swapped);

	// miscelaneous
	CAPTURY_DLL_EXPORT uint64_t liveGetTime();
	CAPTURY_DLL_EXPORT double liveGetFramerate(int liveShot);
	CAPTURY_DLL_EXPORT int liveReconstructRays(int num, const CapturyCornerDetection* corners, float* output, int liveShot);

	// settings
	CAPTURY_DLL_EXPORT int liveStoreString(pluginInfo* plugin, const char* name, const char* value);
	CAPTURY_DLL_EXPORT int liveStoreInt(pluginInfo* plugin, const char* name, int value);
	CAPTURY_DLL_EXPORT int liveStoreDouble(pluginInfo* plugin, const char* name, double value);
	CAPTURY_DLL_EXPORT char* liveRetrieveString(pluginInfo* plugin, const char* name, char* str, int size, const char* defaultValue);
	CAPTURY_DLL_EXPORT int liveRetrieveInt(pluginInfo* plugin, const char* name, int defaultValue);
	CAPTURY_DLL_EXPORT double liveRetrieveDouble(pluginInfo* plugin, const char* name, double defaultValue);


	// annotations
	CAPTURY_DLL_EXPORT int liveAddAnnotation(uint64_t actorId, int frame, const char* name); // return -1 if failed
	CAPTURY_DLL_EXPORT int liveGetAnnotation(uint64_t actorId, int frame, char** an); // return -1 if failed
	CAPTURY_DLL_EXPORT int liveRemoveAnnotation(uint64_t actorId, int frame); // return -1 if failed
	CAPTURY_DLL_EXPORT int liveClearAnnotations(uint64_t actorId); // return -1 if failed

	// ui modification
	CAPTURY_DLL_EXPORT void liveShowToolbar(int yes); // show or hide the main toolbar
	CAPTURY_DLL_EXPORT void liveShowMenubar(int yes); // show or hide the main menu
	CAPTURY_DLL_EXPORT void liveEnableShotManagementMinimalMode(int yes); // show or hide the main menu

	// knobs
	// you can cast the return value into QWidget* and do your own layout and knobs or use the functions below for more convenient knobbing.
	CAPTURY_DLL_EXPORT void* liveAddPanel(pluginInfo* plugin);
	CAPTURY_DLL_EXPORT void livePanelDockLeftRight(pluginInfo* plugin); // the default
	CAPTURY_DLL_EXPORT void livePanelDockBottom(pluginInfo* plugin);
	CAPTURY_DLL_EXPORT void livePanelAddRibbonTab(pluginInfo* plugin);
	CAPTURY_DLL_EXPORT void livePanelAddButton(pluginInfo* plugin, const char* name, const char* label);
	CAPTURY_DLL_EXPORT void livePanelAddSpinBox(pluginInfo* plugin, const char* name, const char* label, int min, int max, int* v, int step);
	CAPTURY_DLL_EXPORT void livePanelAddDoubleSpinBox(pluginInfo* plugin, const char* name, const char* label, double min, double max, double* v, double step);
	CAPTURY_DLL_EXPORT void livePanelAddCheckBox(pluginInfo* plugin, const char* name, const char* label, int* isChecked);
	CAPTURY_DLL_EXPORT void livePanelAddLabel(pluginInfo* plugin, const char* name, const char* label);
	CAPTURY_DLL_EXPORT void livePanelAddSeparator(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT void livePanelAddTextEdit(pluginInfo* plugin, const char* name, const char* label);
	CAPTURY_DLL_EXPORT void livePanelAddLineEdit(pluginInfo* plugin, const char* name, const char* label);

	CAPTURY_DLL_EXPORT int livePanelGetSpinBoxValue(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT double livePanelGetDoubleSpinBoxValue(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT int livePanelIsCheckBoxChecked(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT void livePanelGetTextEditText(pluginInfo* plugin, const char* name, char* buffer, int size);
	CAPTURY_DLL_EXPORT void livePanelGetLineEditText(pluginInfo* plugin, const char* name, char* buffer, int size);

	CAPTURY_DLL_EXPORT void livePanelSetSpinBoxValue(pluginInfo* plugin, const char* name, int v);
	CAPTURY_DLL_EXPORT void livePanelSetDoubleSpinBoxValue(pluginInfo* plugin, const char* name, double v);
	CAPTURY_DLL_EXPORT void livePanelSetCheckBoxChecked(pluginInfo* plugin, const char* name, int yes);
	CAPTURY_DLL_EXPORT void livePanelSetButtonText(pluginInfo* plugin, const char* name, const char* text);
	CAPTURY_DLL_EXPORT void livePanelSetLineEditText(pluginInfo* plugin, const char* name, const char* text);

	CAPTURY_DLL_EXPORT void livePanelAddList(pluginInfo* plugin, const char* name, const char* label);
	CAPTURY_DLL_EXPORT void livePanelAddListItem(pluginInfo* plugin, const char* listName, const char* itemText, int index);	// -1 means append
	CAPTURY_DLL_EXPORT void livePanelDeleteListItem(pluginInfo* plugin, const char* listName, int index);
	CAPTURY_DLL_EXPORT void livePanelClearList(pluginInfo* plugin, const char* listName);
	CAPTURY_DLL_EXPORT int livePanelGetSelectedListItem(pluginInfo* plugin, const char* listName);

	CAPTURY_DLL_EXPORT void livePanelAddComboBox(pluginInfo* plugin, const char* name, const char* label, int isEditable);
	CAPTURY_DLL_EXPORT void livePanelAddComboBoxItem(pluginInfo* plugin, const char* comboboxName, const char* itemText);
	CAPTURY_DLL_EXPORT void livePanelGetComboBoxText(pluginInfo* plugin, const char* name, char* buffer, int size);

//	void livePanelEnable(pluginInfo* plugin, const char* name);
//	int livePanelIsEnabled(pluginInfo* plugin, const char* name);

//	void livePanelSetLabel(pluginInfo* plugin, const char* name, const char* label);

	CAPTURY_DLL_EXPORT void livePanelSetSpinBoxMinMax(pluginInfo* plugin, const char* name, int min, int max);
	CAPTURY_DLL_EXPORT void livePanelSetDoubleSpinBoxMinMax(pluginInfo* plugin, const char* name, double min, double max);

	CAPTURY_DLL_EXPORT void livePanelHideWidget(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT void livePanelShowWidget(pluginInfo* plugin, const char* name);
	CAPTURY_DLL_EXPORT int livePanelIsWidgetHidden(pluginInfo* plugin, const char* name); // 1 if hidden, 0 if visible

#ifdef __cplusplus
}
#endif

