#ifdef WIN32
#pragma warning(disable : 4200)
#pragma warning(disable : 4996)
#define _CRT_SECURE_NO_WARNINGS
#include <Winsock2.h>
#endif

#define RAD2DEGf		(57.29577951308232088f)

#include <vector>
#include <string>
#include <stdio.h>
#include <map>
#include <list>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "captury/plugin.h"

#include <algorithm>
#include <cmath>

#ifdef WIN32
#include <time.h>
#define MSG_CONFIRM 0
#define MSG_DONTWAIT 0
#define socklen_t int
#define ssize_t int
#else
#include <unistd.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#define SOCKET int
#define closesocket close
#endif

#ifdef min
#undef min
#undef max
#endif

#ifndef nullptr
#ifdef WIN32
#define nullptr NULL
#else
#define nullptr __null
#endif
#endif

#ifdef WIN32
static HANDLE thread;
#else
static pthread_t thread;
#endif

#ifdef WIN32
static CRITICAL_SECTION mutex; // lock access to requests
#define initMutex(m)	InitializeCriticalSection(&(m))
#define lockMutex(m)	EnterCriticalSection(&(m))
#define unlockMutex(m)	LeaveCriticalSection(&(m))
#else
static pthread_mutex_t mutex; // lock access to requests
#define initMutex(m)	m = PTHREAD_MUTEX_INITIALIZER
#define lockMutex(m)	pthread_mutex_lock(&(m))
#define unlockMutex(m)	pthread_mutex_unlock(&(m))
#endif

static TCLogHostSuite* logging;

static pluginInfo* plugin;


static volatile bool stopServer = false;

static int port = 2110;
static const int numPorts = 5;

static int useLive = 0;

static std::vector<SOCKET> listenSocks;
static std::vector<std::vector<SOCKET>> clientSocks;

static std::map<int, std::string> headers; // precomputed BVH headers

static std::map<int, CapturyActor*> actors;
static std::vector<int> actorIds;


static void generateHeader(int index);
static bool sendHeader(int index, SOCKET clientSock);

#define COORD(r,c)	((r)*4 + (c))

static float* matrixToXYZ(const float* m4x4, float* euler)
{
	euler[0] = std::atan2(m4x4[COORD(1, 2)], m4x4[COORD(2, 2)]);

	float c2 = std::sqrt(m4x4[COORD(0, 0)]*m4x4[COORD(0, 0)] + m4x4[COORD(0, 1)]*m4x4[COORD(0, 1)]);
	if (euler[0] > 0.0f) {
		euler[0] -= M_PI;
		euler[1] = -std::atan2(-m4x4[COORD(0, 2)], -c2) * RAD2DEGf;
	} else
		euler[1] = -std::atan2(-m4x4[COORD(0, 2)], c2) * RAD2DEGf;
	float s1 = std::sin(euler[0]);
	float c1 = std::cos(euler[0]);
	euler[2] = -std::atan2(s1 * m4x4[COORD(2, 0)] - c1 * m4x4[COORD(1, 0)], c1 * m4x4[COORD(1, 1)] - s1 * m4x4[COORD(2, 1)]) * RAD2DEGf;

	euler[0] = -euler[0] * RAD2DEGf;

	return euler;
}

static void newPose(const CapturyPose* pose, int trackingQuality)
{
	int id = pose->actor;
	int index = -1;
	uint num = liveGetNumActors(useLive);
	if (actorIds.size() == num) {
		for (int i = 0; i < (int)actorIds.size(); ++i) {
			if (actorIds[i] == id) {
				index = i;
				break;
			}
		}
	}
	if (index == -1) {
		std::vector<uint64_t> ids(num);
		actorIds.clear();
		liveGetActorIds(num, &ids[0], useLive);
		int at = 0;
		logging->info("BVH: now have %d actors:", num);
		for (int i = 0; i < (int)ids.size(); ++i) {
			LiveActorMode mode = liveGetActorMode(ids[i], useLive);
//			logging->info("BVH streaming: found actor id %d %s mode %d", ids[i], useLive ? "live" : "playback", mode);
			if (mode != liveSkeletonScaling && mode != liveSkeletonTracking && mode != liveSkeletonPlayback)
				continue;

			logging->info("BVH: actor %x is active", ids[i]);
			actorIds.push_back(ids[i]);
			if (ids[i] == id)
				index = at;
			++at;
		}

		if (index == -1) { // weird still not there
			//logging->info("BVH streaming: still no actor %d.", id);
			return;
		}
	}

	lockMutex(mutex);
	if (index < 0 || index >= (int)clientSocks.size()) {
		logging->warning("BVH streaming: found actor %d that doesn't have a port. only have ports %d - %d", index, port, port + (int)clientSocks.size() - 1);
		unlockMutex(mutex);
		return;
	}
	std::vector<SOCKET> socks = clientSocks[index];
	unlockMutex(mutex);

	if (headers[id].empty() || actors.count(id) == 0) {// no header yet
		generateHeader(id);

		if (headers[id].empty()) // failed to generate header...
			return;

		// tell everyone about the new skeleton
		for (int i = 0; i < (int)socks.size(); ++i)
			sendHeader(index, socks[i]);
	}

	struct Matrix {
		float vals[16];
	};

	CapturyActor* actor;
	if (actors.count(id) == 0) {
		actor = liveGetActor(id, useLive);
		if (actor == nullptr)
			return;

		actors[actorIds[index]] = actor;
	} else
		actor = actors[id];

	// generate pose
	std::string motion;
	std::vector<Matrix> mats(pose->numTransforms);
	for (int i = 0; i < pose->numTransforms; ++i) {
		liveTransformToMatrix(&pose->transforms[i], &mats[i].vals[0]);
		float swapped[16];
		memcpy(swapped, &mats[i].vals[0], sizeof(float)*16);
//		liveSwapHandedness(&mats[i].vals[0], swapped);
		float euler[3];
		char buf[200];

		bool hasChild = false;
		for (int n = i+1; n < actor->numJoints; ++n) {
			if (actor->joints[n].parent == i) {
				hasChild = true;
				break;
			}
		}

		if (!hasChild)
			continue;

		if (i == 0) {
			//liveMatrixToEuler(&swapped[0], euler);

			float xyz[3];
			matrixToXYZ(swapped, xyz);
			// Matrix4f eigenMat(&swapped[0]);
			// Vector3f ea = eigenMat.topLeftCorner<3, 3>().eulerAngles(0, 1, 2) * RAD2DEGf;
			// logging->info("%g %g %g    %g %g %g", ea.x(), ea.y(), ea.z(), xyz[0], xyz[1], xyz[2]);

			// sprintf(buf, "%g %g %g %g %g %g", swapped[3], swapped[7], swapped[11], euler[0], euler[1], euler[2]);
			sprintf(buf, "%g %g %g %g %g %g", swapped[3], swapped[7], swapped[11], xyz[0], xyz[1], xyz[2]);
			//			sprintf(buf, "%g %g %g %g %g %g", 0.0f, 0.0f, 0.0f, euler[0], euler[1], euler[2]);
			motion.append(buf);
		} else {
			float tmp[16];
			int parent = actor->joints[i].parent;
			liveMatrixMatrix(&mats[parent].vals[0], &mats[i].vals[0], tmp);
			matrixToXYZ(tmp, euler);
			// Matrix4f eigenMat(&tmp[0]);
			// Vector3f ea = eigenMat.topLeftCorner<3, 3>().eulerAngles(0, 1, 2) * RAD2DEGf;
			//sprintf(buf, " %g %g %g", 0.0f, 0.0f, 0.0f);
			// sprintf(buf, " %g %g %g", euler[0], euler[1], euler[2]);
			sprintf(buf, " %g %g %g", euler[0], euler[1], euler[2]);
			motion.append(buf);
		}
		liveInvertMatrix(swapped, &mats[i].vals[0]);
	}
	motion.append("\n");

	for (int i = 0; i < (int)socks.size(); ++i) {
		if (send(socks[i], &motion[0], motion.size(), MSG_DONTWAIT | MSG_NOSIGNAL) < 0) {
			switch (errno) {
			case ECONNRESET:
			case EPIPE:
				lockMutex(mutex);
				for (int n = 0; n < (int)clientSocks[index].size(); ++n) {
					if (clientSocks[index][n] == socks[i]) {
						logging->info("BVH streaming: closing socket %d.%d", index, n);
						closesocket(clientSocks[index][i]);
						clientSocks[index].erase(clientSocks[index].begin()+i);
						unlockMutex(mutex);
						return;
					}
				}
				unlockMutex(mutex);
				// fall through
			default:
				logging->warning("BVH streaming: failed to send pose %d: %s", errno, strerror(errno));
				break;
			}
		}
	}
}

static void actorModeChanged(uint64_t actorId, LiveActorMode mode)
{
	switch (mode) {
	case liveSkeletonPreloaded:
		// gone - we'll never see this one again
		break;
	case liveSkeletonInactive:
	case liveSkeletonSearching:
	case liveSkeletonSearchingForRecolor:
		// stopped
		break;
	case liveSkeletonScaling:
	case liveSkeletonTracking:
	case liveSkeletonPlayback:
		// active
		break;
	}
}

static void appendChildren(std::string& header, CapturyActor* actor, int parent, char* buf, int indent)
{
	for (int i = parent + 1; i < actor->numJoints; ++i) {
		if (actor->joints[i].parent != parent)
			continue;

		char indnt[100];
		for (int n = 0; n < indent; ++n)
			indnt[n] = '\t';
		indnt[indent] = '\0';
		CapturyJoint* j = &actor->joints[i];

		bool hasChild = false;
		for (int n = i+1; n < actor->numJoints; ++n) {
			if (actor->joints[n].parent == i) {
				hasChild = true;
				break;
			}
		}

		if (hasChild) {
			sprintf(buf, "%sJOINT %s\n%s{\n%s\tOFFSET\t%g\t%g\t%g\n%s\tCHANNELS 3  Xrotation Yrotation Zrotation\n", indnt, j->name, indnt, indnt, j->offset[0], j->offset[1], j->offset[2], indnt);
			header.append(buf);
			appendChildren(header, actor, i, buf, indent + 1);
			header.append(indnt);
			header.append("}\n");
		} else {
			sprintf(buf, "%sEnd Site\n%s{\n%s\tOFFSET\t%g\t%g\t%g\n%s}\n", indnt, indnt, indnt, j->offset[0], j->offset[1], j->offset[2], indnt);
			header.append(buf);
		}
	}
}

static void generateHeader(int id)
{
	if (!headers[id].empty()) { // already generated
		logging->info("BVH streaming: NOT generating header %d. alredy exists.", id);
		return;
	}

	// make sure we have the actor
	CapturyActor* actor;
	if (actors.count(id) == 0) {
		actor = liveGetActor(id, useLive);
		if (actor == nullptr)
			return;

		actors[id] = actor;
	} else
		actor = actors[id];

	logging->info("BVH streaming: generating header %d", id);

	std::string header("HIERARCHY\n");
	char buf[1000];
	sprintf(buf, "ROOT %s\n{\n\tOFFSET\t0\t0\t0\n\tCHANNELS 6  Xposition Yposition Zposition Xrotation Yrotation Zrotation\n", actor->joints[0].name);
	header.append(buf);
	appendChildren(header, actor, 0, buf, 1);
	sprintf(buf, "}\nMOTION\nFrames: 0\nFrame Time: %g\n", 1.0 / liveGetFramerate(useLive));
	header.append(buf);

	headers[id] = header;
}

static bool sendHeader(int id, SOCKET clientSock)
{
	if (headers[id].empty())
		generateHeader(id);

	if (!headers[id].empty()) {
		if (send(clientSock, &headers[id][0], headers[id].size(), MSG_DONTWAIT | MSG_NOSIGNAL) < 0) {
			closesocket(clientSock);
			logging->warning("BVH streaming: failed to send header. %d: %s", errno, strerror(errno));
			return false;
		}
		logging->info("BVH streaming: sent header %d to %d", id, clientSock);
	}

	return true;
}

// listen for new connections
#ifdef WIN32
static DWORD WINAPI runServer(void* x)
#else
static void* runServer(void* x)
#endif
{
#ifdef WIN32
	WSADATA init;
	const int ret = WSAStartup(WINSOCK_VERSION, &init);
#endif

	// create listening sockets
	for (int i = 0; i < numPorts; ++i) {
		SOCKET sock = socket(AF_INET, SOCK_STREAM | SOCK_NONBLOCK, 0);
		if (sock == -1) {
			logging->warning("BVH streaming: Socket creation failed.");
			return 0;
		}

		int value = 1;
		if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&value, sizeof(value)) == -1) {
			logging->warning("BVH streaming: cannot reuse socket address.");
			closesocket(sock);
			return nullptr;
		}

		struct sockaddr_in addr;
		addr.sin_family = AF_INET;
		addr.sin_port = htons(port+i);
		addr.sin_addr.s_addr = INADDR_ANY;

		if (bind(sock, (struct sockaddr*) &addr, sizeof(addr)) < 0) {
			logging->warning("BVH streaming: Socket cannot bind to port %d.", port);
			closesocket(sock);
			return nullptr;
		}

		listen(sock, 3);

		listenSocks.push_back(sock);
	}

	// listen for connections
	while (!stopServer) {
		fd_set set;
		struct timeval timeout;
		FD_ZERO(&set); /* clear the set */
		SOCKET maxSock = listenSocks[0];
		for (int i = 0; i < numPorts; ++i) {
			FD_SET(listenSocks[i], &set); /* add our file descriptor to the set */
			if (listenSocks[i] > maxSock)
				maxSock = listenSocks[i];
		}

		timeout.tv_sec = 1;
		timeout.tv_usec = 0;

		int rv = select(maxSock + 1, &set, NULL, NULL, &timeout);
		if(rv == -1) { // error
			logging->warning("BVH streaming: select failed %d: %s", errno, strerror(errno));
			continue;
		} else if(rv == 0) // timeout
			continue;

		for (int i = 0; i < numPorts; ++i) {
			if (!FD_ISSET(listenSocks[i], &set))
				continue;

			sockaddr_in clientAddr;
			socklen_t len = sizeof(clientAddr);
			SOCKET clientSock = accept(listenSocks[i], (struct sockaddr *) &clientAddr, &len);
			if (clientSock < 0) {
				logging->warning("BVH streaming: failed to accept connection");
				continue;
			}

			logging->info("BVH streaming: accepted socket %d from %d.%d.%d.%d", i, clientAddr.sin_addr.s_addr & 0xFF, (clientAddr.sin_addr.s_addr & 0xFF00) >> 8, (clientAddr.sin_addr.s_addr & 0xFF0000) >> 16, (clientAddr.sin_addr.s_addr & 0xFF000000) >> 24);

			if (sendHeader(actorIds[i], clientSock)) {
				lockMutex(mutex);
				clientSocks[i].push_back(clientSock);
				unlockMutex(mutex);
			}
		}

	}

	// cleanup
	for (int i = 0; i < numPorts; ++i)
		closesocket(listenSocks[i]);

#ifdef WIN32
	WSACleanup();
#endif
	return 0;
}

static void activate(pluginInfo* info)
{
	clientSocks.resize(numPorts);

	stopServer = false;
//	printf("activating RemoteCaptury\n");
#ifdef WIN32
	thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)runServer, 0, 0, NULL);
#else
	pthread_create(&thread, nullptr, runServer, nullptr);
#endif

	liveRegisterPoseCallback(newPose);
}

static void deactivate()
{
	stopServer = true;
#ifdef WIN32
	WaitForSingleObject(thread, 1000); // wait for one second
#else
	void* retval;
	pthread_join(thread, &retval);
#endif
	liveUnregisterPoseCallback(newPose);
}

static void deinit(pluginInfo* info)
{
	stopServer = true;

#ifdef WIN32
	WaitForSingleObject(thread, 1000); // wait for one second
#else
	void* retval;
	pthread_join(thread, &retval);
#endif

//	printf("deiniting RemoteCaptury\n");
}

static void knobChanged(const char* name)
{
	if (strcmp(name, "port") == 0) {
		port = livePanelGetSpinBoxValue(plugin, name);
		// TODO restart server - cannot do that now because it will hang
//		deactivate();
//		activate(plugin);
	} else if (strcmp(name, "useLive") == 0) {
		useLive = livePanelIsCheckBoxChecked(plugin, name);
	}
}

static uint getConstraints(uint size, CapturyConstraint* constraints)
{
/*	if (constraints.empty())
		return 0;

	const uint totalSize = 12 + 4 + constraints.size() * (1 + 20 + 20 + 12 + 4 + 16 + 4);

	int bufferType = 7; // constraintsBufferType
	timestamp_t timestamp = 0;

	uint at = 0;
	memcpy(&buffer[0], &bufferType, 4);
	memcpy(&buffer[4], &timestamp, 8);
	at+=12;

	*((int*)&buffer[at]) = constraints.size();
	at += 4;

	for (uint i = 0; i < constraints.size(); ++i) {
		buffer[at] = constraints[i].constrType;
		++at;
		*((int*)&buffer[at]) = constraints[i].originActor;
		at += 4;
		*((int*)&buffer[at]) = liveMapJoint(constraints[i].originActor, constraints[i].originJoint);
		at += 4;
		memcpy(&buffer[at], constraints[i].originOffset, 12);
		at += 12;

		*((int*)&buffer[at]) = constraints[i].targetActor;
		at += 4;
		*((int*)&buffer[at]) = liveMapJoint(constraints[i].targetActor, constraints[i].targetJoint);
		at += 4;
		memcpy(&buffer[at], constraints[i].targetOffset, 12);
		at += 12;

		memcpy(&buffer[at], constraints[i].targetVector, 12);
		at += 12;
		*((float*)&buffer[at]) = constraints[i].targetValue;
		at += 4;
		memcpy(&buffer[at], constraints[i].targetRotation, 16);
		at += 16;

		*((float*)&buffer[at]) = constraints[i].weight;
		at += 4;
	}

	// clear the constraints
	constraints.clear();
*/
	return 0;
}

extern "C" unsigned char initPlugin(pluginInfo* info)
{
	if (info->size != sizeof(pluginInfo)) {
		logging->warning("incompatible version of CapturyLive for plugin");
		return 0;
	}

	plugin = info;

	info->name = "BVH";
	info->activate = activate;
	info->knobChanged = knobChanged;
	info->deactivate = deactivate;
	info->deinit = deinit;
	info->getConstraints = getConstraints;
	info->actorModeChanged = actorModeChanged;

	info->activeInLiveMode = 1;
	info->activeInPlaybackMode = 1;
	info->defaultDockArea = DOCK_RIGHT;

	logging = (TCLogHostSuite*)info->getHostSuite("Log");

	liveAddPanel(info);

	livePanelAddSpinBox(info, "port", "Port:", 2110, 65535, &port, 1);

//	livePanelAddSpinBox(info, "numPorts", "Number of Ports:", 5, 10, &numPorts, 1);

	livePanelAddCheckBox(info, "useLive", "Use Live Shot", &useLive);

	return 1;
}
