#include <stdio.h>
#include <string.h>
#include "captury/plugin.h"

// this demonstrates the use of the builtin interface for creating a custom panel

// it is a good idea to keep the plugin info struct around
static pluginInfo* plugin;

// this function is called when the plugin gets activated - normally on startup of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void activate(pluginInfo* info)
{
	liveLogInfo("activating Hello world plugin");
}

// this function is called when the plugin gets deactivated - normally on shutdown of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void deactivate()
{
	liveLogInfo("deactivating Hello world plugin");
}

// this function is called when the plugin gets unloaded - normally on shutdown of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void deinit(pluginInfo* info)
{
	liveLogInfo("deiniting Hello world plugin");
}

void knobChanged(const char* name) // name of knob that changed
{
	if (strcmp(name, "hideToolbar") == 0) {
		liveShowToolbar(false);
		liveShowMenubar(false);
		livePanelHideWidget(plugin, "hideToolbar");
		livePanelShowWidget(plugin, "showToolbar");
	} else if (strcmp(name, "showToolbar") == 0) {
		liveShowToolbar(true);
		liveShowMenubar(true);
		livePanelHideWidget(plugin, "showToolbar");
		livePanelShowWidget(plugin, "hideToolbar");
	} else if (strcmp(name, "spinner") == 0) {
		int v = livePanelGetSpinBoxValue(plugin, name);
		liveLogInfo("spinner has value %d now", v);
	} else if (strcmp(name, "setPath") == 0) {
		liveSetProjectDirectory("/home/hasler/recordings");
	}
}

extern "C" unsigned char initPlugin(pluginInfo* info)
{
	if (info->size != sizeof(pluginInfo)) {
		liveLogWarning("incompatible version of CapturyLive for plugin");
		return 0;
	}

	liveLogInfo("initing Hello world plugin");

	info->name = "Hello World panel";
	info->activate = activate; // if you don't want to handle the following events simply don't set a function
	info->deactivate = deactivate;
	info->getConstraints = NULL;
	info->deinit = deinit;
	info->knobChanged = knobChanged; // called when a button is pressed or a knob value changes

	// should the panel be shown in live mode?
	info->activeInLiveMode = 1;
	// should it be shown in playback mode?
	info->activeInPlaybackMode = 1;
	// where do you want it to be docked?
	info->defaultDockArea = DOCK_RIGHT;

	plugin = info; // keep the pluginInfo around

	// add a panel
	liveAddPanel(info);

	// register panel in ribbon
	livePanelAddRibbonTab(info);

	livePanelAddButton(info, "setPath", "set path");

	livePanelAddButton(info, "hideToolbar", "hide toolbar");
	livePanelAddButton(info, "showToolbar", "show toolbar");
	livePanelHideWidget(info, "showToolbar");
	int initValue = 13;
	livePanelAddSpinBox(info, "spinner", "some value", -10, 100, &initValue, 1); // initValue is updated with the value that is stored in the settings

	return 1; // all good
}
