#include <stdio.h>
#include <string.h>
#include "captury/plugin.h"
#include "PluginHelloWorldWithQt.h"

// this demonstrates the use of the builtin interface for creating a custom panel

// it is a good idea to keep the plugin info struct around
static pluginInfo* plugin;

HelloWorld* hw;

// this function is called when the plugin gets activated - normally on startup of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void activate(pluginInfo* info)
{
	liveLogInfo("activating Hello world plugin");
}

// this function is called when the plugin gets deactivated - normally on shutdown of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void deactivate()
{
	liveLogInfo("deactivating Hello world plugin");
}

// this function is called when the plugin gets unloaded - normally on shutdown of CapturyLive
// in the future activation/deactivation may be decoupled from loading/unloading of the library
static void deinit(pluginInfo* info)
{
	liveLogInfo("deiniting Hello world plugin");
}

extern "C" unsigned char initPlugin(pluginInfo* info)
{
	if (info->size != sizeof(pluginInfo)) {
		liveLogWarning("incompatible version of CapturyLive for plugin");
		return 0;
	}

	liveLogInfo("initing Hello world plugin");

	info->name = "Hello Qt World";
	info->activate = activate; // if you don't want to handle the following events simply don't set a function
	info->deactivate = deactivate;
	info->deinit = deinit;

	// should the panel be shown in live mode?
	info->activeInLiveMode = 1;
	// should it be shown in playback mode?
	info->activeInPlaybackMode = 1;
	// where do you want it to be docked?
	info->defaultDockArea = DOCK_RIGHT;

	plugin = info; // keep the pluginInfo around

	// add a panel
	QWidget* panel = (QWidget*)liveAddPanel(info);

	hw = new HelloWorld(panel);

	// register panel in ribbon
	livePanelAddRibbonTab(info);

	return 1; // all good
}

HelloWorld::HelloWorld(QWidget* panel)
{
	ui.setupUi(panel);

	connect(ui.okButton, SIGNAL(clicked()), this, SLOT(okClicked()));
	connect(ui.spinner, SIGNAL(valueChanged(int)), this, SLOT(spinnerChanged(int)));
}

void HelloWorld::okClicked()
{
	liveLogInfo("button 'do it' was pressed");
}

void HelloWorld::spinnerChanged(int v)
{
	liveLogInfo("spinner has value %d now", v);
}
